namespace Ecng.Trading.Xaml
{
	using System;
	using System.Collections.Generic;
	using System.Windows.Threading;

	using Ecng.Common;
	using Ecng.ComponentModel;
	using Ecng.Trading.Algo.Candles;
	using Ecng.Trading.BusinessEntities;

	/// <summary>
	///   .   <see cref="ICandleManager"/>    ,
	///        GUI .
	/// </summary>
	/// <typeparam name="T">  ,   .</typeparam>
	public class GuiCandleManager<T> : GuiObject, ICandleManager
		where T : ICandleManager
	{
		/// <summary>
		///    .          1 .
		/// </summary>
		/// <param name="candleManager">   .</param>
		public GuiCandleManager(T candleManager)
		{
			this.CandleManager = candleManager;
		}

		/// <summary>
		///    .          1 .
		/// </summary>
		/// <param name="candleManager">   .</param>
		/// <param name="dispatcher">     GUI .</param>
		public GuiCandleManager(T candleManager, Dispatcher dispatcher)
			: base(dispatcher)
		{
			this.CandleManager = candleManager;
		}

		/// <summary>
		///    .
		/// </summary>
		/// <param name="candleManager">   .</param>
		/// <param name="dispatcher">     GUI .</param>
		/// <param name="interval">       .</param>
		public GuiCandleManager(T candleManager, Dispatcher dispatcher, TimeSpan interval)
			: base(dispatcher, interval)
		{
			this.CandleManager = candleManager;
		}

		private T _candleManager;

		/// <summary>
		///    .
		/// </summary>
		public T CandleManager
		{
			get { return _candleManager; }
			private set
			{
				if (value.IsNull())
					throw new ArgumentNullException();

				_candleManager = value;

				_candleManager.NewCandles += NewCandlesHandler;
				_candleManager.CandlesChanged += CandlesChangedHandler;
				_candleManager.CandlesFinished += CandlesFinishedHandler;
				_candleManager.ProcessDataError += ProcessDataErrorHandler;
			}
		}

		#region NewCandles

		/// <summary>
		///    .
		/// </summary>
		public event Action<CandleToken, IEnumerable<Candle>> NewCandles;

		private void NewCandlesHandler(CandleToken token, IEnumerable<Candle> candles)
		{
			base.EventDispatcher.Add(() => NewCandles.SafeInvoke(token, candles));
		}

		#endregion

		#region CandlesChanged

		/// <summary>
		///   .
		/// </summary>
		public event Action<CandleToken, IEnumerable<Candle>> CandlesChanged;

		private void CandlesChangedHandler(CandleToken token, IEnumerable<Candle> candles)
		{
			base.EventDispatcher.Add(() => CandlesChanged.SafeInvoke(token, candles));
		}

		#endregion

		#region CandlesFinished

		/// <summary>
		///    .
		/// </summary>
		public event Action<CandleToken, IEnumerable<Candle>> CandlesFinished;

		private void CandlesFinishedHandler(CandleToken token, IEnumerable<Candle> candles)
		{
			base.EventDispatcher.Add(() => CandlesFinished.SafeInvoke(token, candles));
		}

		#endregion

		#region ProcessDataError

		/// <summary>
		///    .
		/// </summary>
		public event Action<Exception> ProcessDataError;

		private void ProcessDataErrorHandler(Exception error)
		{
			base.EventDispatcher.Add(() => ProcessDataError.SafeInvoke(error));
		}

		#endregion

		/// <summary>
		///    , -  .
		/// </summary>
		/// <param name="security">,     .</param>
		/// <param name="timeFrame">-,     .</param>
		/// <param name="time"> .</param>
		/// <returns>  (null,       ).</returns>
		public TimeFrameCandle GetTimeFrameCandle(Security security, TimeSpan timeFrame, DateTime time)
		{
			return this.CandleManager.GetTimeFrameCandle(security, timeFrame, time);
		}

		/// <summary>
		///    , ,    .
		/// </summary>
		/// <typeparam name="TCandle"> .</typeparam>
		/// <typeparam name="TArg"> .</typeparam>
		/// <param name="security">,     .</param>
		/// <param name="arg">  . ,  <see cref="TimeFrameCandle"/>   <see cref="TimeFrameCandle.TimeFrame"/>.</param>
		/// <param name="candleCount"> ,   .</param>
		/// <returns> .</returns>
		public IEnumerable<TCandle> GetCandles<TCandle, TArg>(Security security, TArg arg, int candleCount)
			where TCandle : Candle, new()
		{
			return this.CandleManager.GetCandles<TCandle, TArg>(security, arg, candleCount);
		}

		/// <summary>
		///    , ,    .
		/// </summary>
		/// <typeparam name="TCandle"> .</typeparam>
		/// <typeparam name="TArg"> .</typeparam>
		/// <param name="security">,     .</param>
		/// <param name="arg">  . ,  <see cref="TimeFrameCandle"/>   <see cref="TimeFrameCandle.TimeFrame"/>.</param>
		/// <param name="timeRange"> ,     .   <see cref="Candle.Time"/>.</param>
		/// <returns> .</returns>
		public IEnumerable<TCandle> GetCandles<TCandle, TArg>(Security security, TArg arg, Range<DateTime> timeRange)
			where TCandle : Candle, new()
		{
			return this.CandleManager.GetCandles<TCandle, TArg>(security, arg, timeRange);
		}

		/// <summary>
		///     ,   .
		/// </summary>
		/// <typeparam name="TCandle"> .</typeparam>
		/// <typeparam name="TArg"> .</typeparam>
		/// <param name="security">,     .</param>
		/// <param name="arg">  . ,  <see cref="TimeFrameCandle"/>   <see cref="TimeFrameCandle.TimeFrame"/>.</param>
		/// <returns>  (null,     ).</returns>
		public TCandle GetLastCandle<TCandle, TArg>(Security security, TArg arg)
			where TCandle : Candle, new()
		{
			return this.CandleManager.GetLastCandle<TCandle, TArg>(security, arg);
		}

		/// <summary>
		///      .
		/// </summary>
		/// <param name="token"> .</param>
		/// <returns>  (null,     ).</returns>
		public Candle GetLastCandle(CandleToken token)
		{
			return this.CandleManager.GetLastCandle(token);
		}

		/// <summary>
		///    .
		/// </summary>
		/// <typeparam name="TCandle"> .</typeparam>
		/// <typeparam name="TArg"> .</typeparam>
		/// <param name="security">,    .</param>
		/// <param name="arg">  . ,  <see cref="TimeFrameCandle"/>   <see cref="TimeFrameCandle.TimeFrame"/>.</param>
		/// <returns>True,  . , false.</returns>
		public bool IsCandlesRegistered<TCandle, TArg>(Security security, TArg arg)
			where TCandle : Candle, new()
		{
			return this.CandleManager.IsCandlesRegistered<TCandle, TArg>(security, arg);
		}

		/// <summary>
		///   .
		/// </summary>
		/// <typeparam name="TCandle"> .</typeparam>
		/// <typeparam name="TArg"> .</typeparam>
		/// <param name="security">,        .</param>
		/// <param name="arg">  . ,  <see cref="TimeFrameCandle"/>   <see cref="TimeFrameCandle.TimeFrame"/>.</param>
		/// <returns> .</returns>
		public CandleToken RegisterCandles<TCandle, TArg>(Security security, TArg arg)
			where TCandle : Candle, new()
		{
			return this.CandleManager.RegisterCandles<TCandle, TArg>(security, arg);
		}

		/// <summary>
		///   .
		/// </summary>
		/// <param name="token"> ,   .</param>
		public void UnRegisterCandles(CandleToken token)
		{
			this.CandleManager.UnRegisterCandles(token);
		}

		/// <summary>
		///   .
		/// </summary>
		protected override void DisposeManaged()
		{
			_candleManager.NewCandles -= NewCandlesHandler;
			_candleManager.CandlesChanged -= CandlesChangedHandler;
			_candleManager.CandlesFinished -= CandlesFinishedHandler;
			_candleManager.ProcessDataError -= ProcessDataErrorHandler;

			base.DisposeManaged();
		}
	}
}